import { Struct } from './struct';
import { load } from './loader';

// Store Loaded Structures
const structures: Record<string, Struct> = {};

// Get Or Load Structure
export function getStructure(name: string) {
    if (structures[name]) {
        // Already Loaded
        return structures[name];
    } else {
        // Load Structure
        try {
            // Create Structure
            const target = new Struct(name);
            structures[name] = target;
            // Parse File
            load(target, name, false);
            // Return
            return target;
        } catch (e) {
            let message = 'Error Loading ' + name;
            if (e instanceof Error) {
                message += ': ' + e.message;
            }
            throw new Error(message);
        }
    }
}